@@ -1,21 +1,21 @@
-## * Allocator (for BuyseTest-options)
+## * Allocator alloc (for BuyseTest-options)
 setGeneric(name = "alloc", 
            def = function(object, ...){standardGeneric("alloc")}
 )
 
-## * Selector (for BuyseTest-options)
+## * Selector select (for BuyseTest-options)
 setGeneric(name = "select", 
            def = function(object, ...){standardGeneric("select")}
 )
 
-## * Selector (for BuyseRes)
+## * Selector getCount (for BuyseRes)
 #' @rdname BuyseRes-getCount
 #' @exportMethod getCount
 setGeneric(name = "getCount",
            def = function(object, type){standardGeneric("getCount")}
 )
 
-## * Selector (for BuyseRes)
+## * Selector getPairScore (for BuyseRes)
 #' @rdname BuyseRes-getPairScore
 #' @exportMethod getPairScore
 setGeneric(name = "getPairScore",
@@ -26,7 +26,7 @@ setGeneric(name = "getPairScore",
            }
            )
 
-## * Selector (for BuyseRes)
+## * Selector getSurvival (for BuyseRes)
 #' @rdname BuyseRes-getSurvival
 #' @exportMethod getSurvival
 setGeneric(name = "getSurvival",
@@ -34,3 +34,12 @@ setGeneric(name = "getSurvival",
                standardGeneric("getSurvival")
            }
 )
+
+## * Selector iid (for BuyseRes)
+#' @rdname BuyseRes-iid
+#' @exportMethod iid
+setGeneric(name = "iid",
+           def = function(object, endpoint = NULL, order = 1){
+               standardGeneric("iid")
+           }
+)
