@@ -44,6 +44,7 @@ setClass(
       DeltaResampling.netBenefit = "matrix",
       DeltaResampling.winRatio = "matrix",
       covariance = "matrix",
+      iid = "list",
       tablePairScore = "list",
       tableSurvival = "list"
       )
@@ -79,6 +80,7 @@ methods::setMethod(
                                    DeltaResampling.netBenefit,
                                    DeltaResampling.winRatio,
                                    covariance,
+                                   iid,
                                    tablePairScore,
                                    tableSurvival,
                                    args){
@@ -135,8 +137,10 @@ methods::setMethod(
                  .Object@DeltaResampling.netBenefit <- DeltaResampling.netBenefit
                  .Object@DeltaResampling.winRatio <- DeltaResampling.winRatio
 
-                 .Object@covariance <- covariance
-
+                 .Object@covariance <- covariance$Sigma
+                 .Object@iid <- list(iid1 = covariance$iid1,
+                                     iid2 = covariance$iid2)
+                 
                  .Object@tablePairScore <- tablePairScore
                  .Object@tableSurvival <- tableSurvival
                  
