@@ -3,9 +3,9 @@
 ## Author: Brice Ozenne
 ## Created: sep 26 2018 (12:57) 
 ## Version: 
-## Last-Updated: okt 16 2018 (21:17) 
+## Last-Updated: jan 15 2019 (10:16) 
 ##           By: Brice Ozenne
-##     Update #: 217
+##     Update #: 337
 ##----------------------------------------------------------------------
 ## 
 ### Commentary: 
@@ -25,11 +25,12 @@
 #' @param sim [function] take two arguments:
 #' the sample size in the control group (\code{n.C}) and the sample size in the treatment group (\code{n.C})
 #' and generate datasets. The datasets must be data.table objects.
-#' @param sample.size the various sample sizes at which the simulation should be perform.
+#' @param sample.size [integer vector, >0] the various sample sizes at which the simulation should be perform.
 #' Disregarded if any of the arguments \code{sample.sizeC} or \code{sample.sizeT} are specified.
-#' @param sample.sizeC the various sample sizes in the control group.
-#' @param sample.sizeT the various sample sizes in the treatment group.
-#' @param n.rep the number of simulations.
+#' @param sample.sizeC [integer vector, >0] the various sample sizes in the control group.
+#' @param sample.sizeT [integer vector, >0] the various sample sizes in the treatment group.
+#' @param n.rep [integer, >0] the number of simulations.
+#' @param null [numeric vector] the null hypothesis to be tested for the net benefit (first element) and the win ratio (second element).
 #' @param cpus [integer, >0] the number of CPU to use.
 #' Only the permutation test can use parallel computation.
 #' Default value read from \code{BuyseTest.options()}.
@@ -43,6 +44,7 @@
 #' @param transformation [logical] should the CI be computed on the logit scale / log scale for the net benefit / win ratio and backtransformed.
 #' Otherwise they are computed without any transformation.
 #' Default value read from \code{BuyseTest.options()}.
+#' @param order.Hprojection [integer 1,2] the order of the H-project to be used to compute the asymptotic variance.
 #' @param ... parameters from \code{BuyseTest}.
 #' 
 
@@ -70,17 +72,21 @@
 ## * powerBuyseTest (code)
 ##' @rdname powerBuyseTest
 ##' @export
-powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT = NULL, n.rep, cpus = 1,                          
-                          alternative = NULL, seed = 10, conf.level = NULL, transformation = NULL, trace = 1, ...){
+powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT = NULL, n.rep, null = c(0,1), cpus = 1,                          
+                           alternative = NULL, seed = 10, conf.level = NULL, order.Hprojection = NULL, transformation = NULL, trace = 1,
+                           ...){
 
     call <- match.call()$sim
-    
+
     ## ** normalize and check arguments
     name.call <- names(match.call())
     option <- BuyseTest.options()
     if(is.null(transformation)){
         transformation <- option$transformation
     }
+    if(is.null(order.Hprojection)){
+        order.Hprojection <- option$order.Hprojection
+    }
     if(is.null(conf.level)){
         conf.level <- option$conf.level
     }
@@ -108,7 +114,15 @@ powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT =
     if(length(sample.sizeT)!=length(sample.sizeC)){
         stop("Arguments \'sample.sizeT\ and \'sample.sizeC\' must have the same length \n")
     }
+    validNumeric(null,
+                 valid.length = 2,
+                 method = "BuyseTest")
+    names(null) <- c("netBenefit","winRatio")
+        
     n.sample.size <- length(sample.sizeT)
+    grid.inference <- expand.grid(order = order.Hprojection,
+                                  transformation = transformation)
+    n.inference <- NROW(grid.inference)
     sample.sizeCmax <- sample.sizeC[n.sample.size]
     sample.sizeTmax <- sample.sizeT[n.sample.size]
 
@@ -121,7 +135,9 @@ powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT =
     ## initialized arguments are stored in outArgs
     outArgs <- initializeArgs(cpus = cpus, option = option, name.call = name.call, alternative = alternative,
                               data = NULL, model.tte = NULL, keep.pairScore = TRUE, ...)
-
+    if(outArgs$method.tte==1 && n.sample.size >1){
+        stop("Peron correction not compatible with powerBuyseTest for more than one sample size \n")
+    }
     if(any(outArgs$operator!=">0")){
         stop("Cannot use argument \'operator\' with powerBuyseTest \n")
     }
@@ -144,13 +160,14 @@ powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT =
         
     ## ** create weights matrix for survival endpoints
     ## WARNING when updating code: names in the c() must precisely match output of initializeData, in the same order
-    outArgs[c("Wscheme","index.survival_M1","threshold.TTE_M1","outSurv")] <- buildWscheme(method.tte = outArgs$method.tte,
-                                                                                           endpoint = outArgs$endpoint,
-                                                                                           D = outArgs$D,
-                                                                                           D.TTE = outArgs$D.TTE,
-                                                                                           n.strata = outArgs$n.strata,
-                                                                                           type = outArgs$type,
-                                                                                           threshold = outArgs$threshold)
+    out.name <- c("Wscheme","endpoint.UTTE","index.UTTE","D.UTTE","reanalyzed","outSurv")
+    outArgs[out.name] <- buildWscheme(method.tte = outArgs$method.tte,
+                                      endpoint = outArgs$endpoint,
+                                      D = outArgs$D,
+                                      D.TTE = outArgs$D.TTE,
+                                      n.strata = outArgs$n.strata,
+                                      type = outArgs$type,
+                                      threshold = outArgs$threshold)
 
     ## ** Display
     if (trace > 1) {
@@ -174,7 +191,7 @@ powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT =
         
     }
     ## ** define environment
-    name.copy <- c("initializeSurvival_Peron", "initializeData", ".BuyseTest", "call", "sim",
+    name.copy <- c("call", "sim", "option",
                    "outArgs", "sample.sizeTmax", "sample.sizeCmax", "n.sample.size",
                    "sample.size", "sample.sizeC", "sample.sizeT", "n.rep", "alternative", "seed")
     envirBT <- new.env()
@@ -184,39 +201,28 @@ powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT =
 
     ## ** warper
     warper <- function(i, envir){
-        iOut <- matrix(NA, nrow = n.sample.size, ncol = 13,
-                       dimnames = list(NULL, c("simulation","n.T","n.C",
+        iOut <- matrix(NA, nrow = n.sample.size * n.inference, ncol = 14,
+                       dimnames = list(NULL, c("simulation","n.T","n.C","method.inference",
                                                "netBenefit","netBenefit.se","netBenefit.lower","netBenefit.upper","netBenefit.p.value",
                                                "winRatio","winRatio.se","winRatio.lower","winRatio.upper","winRatio.p.value")))
+        iOut <- as.data.frame(iOut)
+        iOut[,"simulation"] <- i
 
-        ## *** Simulate data
-        envir$outArgs$data <- do.call(eval(envir$call), args = list(n.T = sample.sizeTmax, n.C = sample.sizeCmax))
-        ## envir$outArgs$data <- envir$sim(n.T = sample.sizeTmax, n.C = sample.sizeCmax)
         ## *** Initialize data
+        out.name <- c("data","M.endpoint","M.censoring",
+                      "index.C","index.T","index.strata",
+                      "index.endpoint","index.censoring","level.treatment","level.strata",
+                      "n.strata","n.obs","n.obsStrata","cumn.obsStrata")
+        envir$outArgs[out.name] <- initializeData(data = do.call(eval(envir$call), args = list(n.T = sample.sizeTmax, n.C = sample.sizeCmax)),
+                                                  type = envir$outArgs$type,
+                                                  endpoint = envir$outArgs$endpoint,
+                                                  method.tte = envir$outArgs$method.tte,
+                                                  censoring = envir$outArgs$censoring,
+                                                  operator = envir$outArgs$operator,
+                                                  strata = envir$outArgs$strata,
+                                                  treatment = envir$outArgs$treatment,
+                                                  copy = FALSE)
 
-        ## convert character/factor to numeric for binary endpoints
-        ## to be done
-        
-        ## convert treatment to binary indicator
-        trt2bin <- setNames(0:1,outArgs$level.treatment)
-        envir$outArgs$data[, c(envir$outArgs$treatment) := trt2bin[as.character(.SD[[1]])], .SDcols = envir$outArgs$treatment]
-
-        ## NA column for fake censoring 
-        envir$outArgs$data[,c("..NA..") := as.numeric(NA)]
-
-        ## rowIndex
-        envir$outArgs$data[,c("..rowIndex..") := 1:.N]
-
-        ## ** export
-        keep.col <- c(envir$outArgs$treatment,"..rowIndex..")
-        if(envir$outArgs$method.tte==1){keep.col <- c(keep.col,envir$outArgs$endpoint[envir$outArgs$type == 3], envir$outArgs$censoring[envir$outArgs$type == 3])}
-
-        envir$outArgs$M.endpoint <- as.matrix(envir$outArgs$data[, .SD, .SDcols = envir$outArgs$endpoint])
-        envir$outArgs$M.censoring <- as.matrix(envir$outArgs$data[, .SD, .SDcols = envir$outArgs$censoring])
-        envir$outArgs$data <- envir$outArgs$data[,.SD,.SDcols = keep.col]
-        envir$outArgs$n.obs <- NROW(envir$outArgs$data)
-        envir$outArgs$n.obsStrata <- envir$outArgs$n.obs
-        
         ## *** Point estimate
         outPoint <- .BuyseTest(envir = envir,
                                keep.pairScore = TRUE,
@@ -231,10 +237,11 @@ powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT =
                                        threshold = envir$outArgs$threshold)
 
         for(iSample in 1:n.sample.size){ ## iSample <- 1
-            iOut[iSample,"simulation"] <- i
-            iOut[iSample,"n.C"] <- envir$sample.sizeC[iSample]
-            iOut[iSample,"n.T"] <- envir$sample.sizeT[iSample]
-
+            iIndex.store <- seq(1 + (iSample-1)*n.inference, iSample*n.inference)
+            iOut[iIndex.store,"n.C"] <- envir$sample.sizeC[iSample]
+            iOut[iIndex.store,"n.T"] <- envir$sample.sizeT[iSample]
+            iPairs <- envir$sample.sizeC[iSample]*envir$sample.sizeT[iSample]
+            
             tableSample <- lapply(tablePairScore, function(iEndpoint){ ## iEndpoint <- tablePairScore[[1]]
                 ## restrict pairs 
                 index <- intersect(which(iEndpoint$indexWithinStrata.C <= envir$sample.sizeC[iSample]),
@@ -249,6 +256,16 @@ powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT =
                 iEndpoint[, c("index.T") := old2new[.SD$index.T]]
                 iEndpoint[, c("index.C") := old2new[.SD$index.C]]
 
+                ## update correction (no strata)
+                if(envir$outArgs$correction.uninf>0 && iSample < n.sample.size && sum(iEndpoint$uninf)>0){
+## new weighting                    
+                        mfactor <- sum(iEndpoint$favorable + iEndpoint$unfavorable + iEndpoint$neutral + iEndpoint$uninf) / sum(iEndpoint$favorable + iEndpoint$unfavorable + iEndpoint$neutral)
+                        iEndpoint[, c("favorableC") :=.SD$favorable * .SD$weight * mfactor]
+                        iEndpoint[, c("unfavorableC") :=.SD$unfavorable * .SD$weight * mfactor]
+                        iEndpoint[, c("neutralC") :=.SD$neutral * .SD$weight * mfactor]
+                    
+                }
+                
                 ## export
                 return(iEndpoint)
             })
@@ -256,37 +273,59 @@ powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT =
             ## *** Point estimate
             MresSample <- do.call(rbind,lapply(tableSample, function(iEndpoint){
                 return(c("npairs" = NROW(iEndpoint),
-                         "favorable" = sum(iEndpoint$favorable),
-                         "unfavorable" = sum(iEndpoint$unfavorable)))
+                         "favorable" = sum(iEndpoint$favorableC),
+                         "unfavorable" = sum(iEndpoint$unfavorableC)))
             }))
-            iOut[iSample,"netBenefit"] <- (sum(MresSample[,"favorable"]) - sum(MresSample[,"unfavorable"]))/MresSample[1,"npairs"]
-            iOut[iSample,"winRatio"] <- sum(MresSample[,"favorable"]) / sum(MresSample[,"unfavorable"])
+            
+            iOut[iIndex.store,"netBenefit"] <- (sum(MresSample[,"favorable"]) - sum(MresSample[,"unfavorable"]))/MresSample[1,"npairs"]
+            iOut[iIndex.store,"winRatio"] <- sum(MresSample[,"favorable"]) / sum(MresSample[,"unfavorable"])
 
             ## *** Inference 
             if(outArgs$method.inference %in% c("asymptotic")){
+                ## warning: only work if no strata, otherwise n.pairs/count.favorable/count.unfavorable needs to be sum over strata
+                ## see BuyseTest.R
                 outCovariance <- inferenceUstatistic(tableSample,
+                                                     order = max(order.Hprojection), ## if order 1 and 2 are requested by the user then feed order 2
                                                      count.favorable = matrix(MresSample[,"favorable"], nrow = 1),
                                                      count.unfavorable = matrix(MresSample[,"unfavorable"], nrow = 1),
-                                                     n.pairs = envir$sample.sizeC[iSample]*envir$sample.sizeT[iSample],
+                                                     n.pairs = iPairs,
                                                      n.C = envir$sample.sizeC[iSample],
                                                      n.T = envir$sample.sizeT[iSample],
+                                                     level.strata = envir$outArgs$level.strata,
                                                      n.strata = envir$outArgs$n.strata,
                                                      n.endpoint = length(envir$outArgs$endpoint),
                                                      endpoint = envir$outArgs$endpoint)
-
-                for(iStatistic in c("netBenefit","winRatio")){
-                    outCI <- confint_Ustatistic(Delta = iOut[iSample,iStatistic],
-                                                pc.favorable = MresSample[,"favorable"]/MresSample[,"npairs"],
-                                                pc.unfavorable =  MresSample[,"unfavorable"]/MresSample[,"npairs"],
-                                                covariance = outCovariance$Sigma, statistic = iStatistic,
-                                                alternative = alternative, alpha = alpha,
-                                                endpoint = envir$outArgs$endpoint, transformation = transformation)
                 
-                    iOut[iSample,paste0(iStatistic,".se")] <- outCI[1,"se"]
-                    iOut[iSample,paste0(iStatistic,".lower")] <- outCI[1,"lower.ci"]
-                    iOut[iSample,paste0(iStatistic,".upper")] <- outCI[1,"upper.ci"]
-                    iOut[iSample,paste0(iStatistic,".p.value")] <- outCI[1,"p.value"]
+                for(iStatistic in c("netBenefit","winRatio")){
+                    for(iInference in 1:n.inference){ ## iInference <- 1
+                        ## cat("\n",iStatistic," ",iInference," ",iSample,"\n")
+                            
+                        iTransformation <- grid.inference[iInference,"transformation"]
+                        iOrder <- grid.inference[iInference,"order"]
+                        if(iOrder == max(order.Hprojection)){
+                            iCovariance <- outCovariance$Sigma                         
+                        }else{
+                            ## recompute the covariance matrix to remove second order term 
+                            iCovariance <- .iid2cov(A.iid = outCovariance$iid1, A2.iid = NULL,
+                                                    order = iOrder, endpoint = envir$outArgs$endpoint, n.endpoint = length(envir$outArgs$endpoint))
+                        }
+                        outCI <- confint_Ustatistic(Delta = iOut[iIndex.store[iInference],iStatistic],
+                                                    pc.favorable = MresSample[,"favorable"]/MresSample[,"npairs"],
+                                                    pc.unfavorable =  MresSample[,"unfavorable"]/MresSample[,"npairs"],
+                                                    covariance = iCovariance, statistic = iStatistic,
+                                                    alternative = alternative, alpha = alpha, null = null[iStatistic],
+                                                    endpoint = envir$outArgs$endpoint, transformation = iTransformation,
+                                                    continuity.correction = envir$option$continuity.correction,
+                                                    n.pairs = iPairs)
+
+                        iOut[iIndex.store[iInference],"method.inference"] <- paste0("order=",iOrder," - transformation=",iTransformation)
+                        iOut[iIndex.store[iInference],paste0(iStatistic,".se")] <- outCI[1,"se"]
+                        iOut[iIndex.store[iInference],paste0(iStatistic,".lower")] <- outCI[1,"lower.ci"]
+                        iOut[iIndex.store[iInference],paste0(iStatistic,".upper")] <- outCI[1,"upper.ci"]
+                        iOut[iIndex.store[iInference],paste0(iStatistic,".p.value")] <- outCI[1,"p.value"]
+                    }
                 }
+            
 
             }
 
@@ -314,31 +353,48 @@ powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT =
                                              })
                                  )
     }else { ## *** parallel permutation test
+        n.block <- max(cpus,round(sqrt(n.rep)))
+        rep.perBlock0 <- max(1,floor(n.rep/n.block))
+        rep.perBlock <- c(rep(rep.perBlock0, n.block-1), n.rep - (n.block-1)*rep.perBlock0)
+        cumsum.rep.perBlock <- c(0,cumsum(rep.perBlock))
 
         ## define cluster
         if(trace>0){
             cl <- suppressMessages(parallel::makeCluster(cpus, outfile = ""))
-            pb <- utils::txtProgressBar(max = n.rep, style = 3)          
+            pb <- utils::txtProgressBar(max = n.block, style = 3)          
         }else{
             cl <- parallel::makeCluster(cpus)
         }
         ## link to foreach
         doParallel::registerDoParallel(cl)
 
+        ## seed
+        if (!is.null(seed)) {
+            set.seed(seed)
+            seqSeed <- sample.int(1e3, size = cpus)
+            parallel::clusterApply(cl, seqSeed, function(x){
+                set.seed(x)
+            })
+        }         
+
         ## export package
         parallel::clusterCall(cl, fun = function(x){
             suppressPackageStartupMessages(library(BuyseTest, quietly = TRUE, warn.conflicts = FALSE, verbose = FALSE))
         })
         ## export functions
-        toExport <- c(".BuyseTest","initializeSurvival_Peron","pairScore2dt","inferenceUstatistic","confint_Ustatistic", "validNumeric")
-
-        i <- NULL ## [:forCRANcheck:] foreach        
+        toExport <- c(".BuyseTest","initializeData","pairScore2dt","inferenceUstatistic","confint_Ustatistic", ".iid2cov", "validNumeric")
+        
+        i <- NULL ## [:forCRANcheck:] foreach
         ls.simulation <- foreach::`%dopar%`(
-                                       foreach::foreach(i=1:n.rep,
-                                                        .export = toExport),                                            
-                                       {                                           
-                                           if(trace>0){utils::setTxtProgressBar(pb, i)}
-                                           return(warper(i = i, envir = envirBT))
+                                      foreach::foreach(i=1:n.block,
+                                                       .export = toExport),                                            
+                                      {                                           
+                                          if(trace>0){utils::setTxtProgressBar(pb, i)}
+                                          ls.out <- list()
+                                          for(j in 1:rep.perBlock[i]){
+                                            ls.out[[j]] <- warper(i = j + cumsum.rep.perBlock[i], envir = envirBT)
+                                          }
+                                          return(do.call(rbind, ls.out))
                       
                                        })
 
@@ -348,12 +404,15 @@ powerBuyseTest <- function(sim, sample.size, sample.sizeC = NULL, sample.sizeT =
     dt.out <- as.data.table(do.call(rbind, ls.simulation))
 
     ## ** export
+    attr(outArgs$method.inference,"continuity.correction") <- option$continuity.correction
+        
     BuyseSim.object <- BuyseSim(
         alternative = alternative,      
-        conf.level = conf.level,      
+        method.inference = outArgs$method.inference,
+        conf.level = conf.level,
+        null = null,
         n.rep = n.rep,      
-        results = dt.out,
-        transformation = transformation
+        results = dt.out
     )
 
     return(BuyseSim.object)
